{% extends 'base.html' %}
{% load propeller %}

{% block title %}
    Propeller Cards
{% endblock %}

{% block content %}
    <div class="row">
    {% propeller_card card1 %}
    </div>
    <div class="row">
    {% propeller_card card2 %}
    </div>
    <div class="row">
    {% propeller_card card3 %}
    </div>
    <div class="row">
    {% propeller_card card4 %}
    </div>
{% endblock %}