{% extends 'base.html' %}

{% load propeller %}

{% block title %}
    Formset
{% endblock %}

{% block content %}
    {% propeller_formset_errors form %}
    <form role="form" method="post">
        {% csrf_token %}
        {% propeller_formset form %}
        {% propeller_button  content="Submit" button_type="submit" %} {% propeller_button content="Reset" button_type="reset" %}
    </form>

{% endblock %}
