# -*- coding: utf-8 -*-

"""Provides expected test results"""

RESULT_CARD1 = '<div class="pmd-card pmd-card-default pmd-z-depth col-md-4">' \
               '<div class="pmd-card-title">' \
               '<div class="media-left">' \
               '<img src="http://propeller.in/assets/images/avatar-icon-40x40.png" width="40" height="40">' \
               '</div>' \
               '<div class="media-body media-middle">' \
               '<h3 class="pmd-card-title-text">Two-line item</h3>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '</div>' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">' \
               '<div class="pmd-card-title">' \
               '<h2 class="pmd-card-title-text">Title goes here</h2>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '<div class="pmd-card-body">' \
               'Cards provide context and an entry point to more robust information and views. ' \
               'Don\'t overload cards with extraneous information or actions.' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">share</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">thumb_up</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">drafts</i>' \
               '</button>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD2 = '<div class="pmd-card pmd-card-default pmd-z-depth col-md-4">' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">' \
               '<div class="pmd-card-title">' \
               '<h2 class="pmd-card-title-text">Title goes here</h2>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">share</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">thumb_up</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">drafts</i>' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD3 = '<div class="pmd-card pmd-card-inverse pmd-z-depth col-md-4">' \
               '<div class="pmd-card-title">' \
               '<div class="media-left">' \
               '<img src="http://propeller.in/assets/images/avatar-icon-40x40.png" width="40" height="40">' \
               '</div>' \
               '<div class="media-body media-middle">' \
               '<h3 class="pmd-card-title-text">Two-line item</h3>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '</div>' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" class="img-responsive">' \
               '<div class="pmd-card-title">' \
               '<h2 class="pmd-card-title-text">Title goes here</h2>' \
               '<span class="pmd-card-subtitle-text">Secondary text</span>' \
               '</div>' \
               '<div class="pmd-card-body">' \
               'Cards provide context and an entry point to more robust information and views. ' \
               'Don\'t overload cards with extraneous information or actions.' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">share</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">thumb_up</i>' \
               '</button>' \
               '<button class="btn btn-sm pmd-btn-fab pmd-btn-flat pmd-ripple-effect btn-primary" type="button">' \
               '<i class="material-icons pmd-sm">drafts</i>' \
               '</button>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD4 = '<div class="pmd-card pmd-card-media-inline pmd-card-default pmd-z-depth col-md-4">' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" width="80" height="80">' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_CARD5 = '<div class="pmd-card pmd-card-media-inline pmd-card-default pmd-z-depth col-md-4">' \
               '<div class="pmd-card-media">' \
               '<div class="media-body"></div>' \
               '<div class="media-right media-middle">' \
               '<img src="http://propeller.in/assets/images/profile-pic.png" width="80" height="80">' \
               '</div>' \
               '</div>' \
               '<div class="pmd-card-actions">' \
               '<button class="btn btn-primary pmd-ripple-effect pmd-btn-default" href="#" type="button">primary' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">Action' \
               '</button>' \
               '<button class="btn btn-default pmd-ripple-effect pmd-btn-default" href="#" type="button">third' \
               '</button>' \
               '</div>' \
               '</div>'

RESULT_NAVBAR1 = '<nav class="navbar navbar-inverse navbar-static pmd-navbar pmd-z-depth">' \
                 '<div class="container-fluid">' \
                 '<div class="navbar-header">' \
                 '<button aria-expanded="false" class="navbar-toggle collapsed" data-toggle="collapse" type="button">' \
                 '<span class="sr-only">Toggle navigation</span>' \
                 '<span class="icon-bar">' \
                 '</span><span class="icon-bar">' \
                 '</span><span class="icon-bar">' \
                 '</span>' \
                 '</button>' \
                 '<a href="https://github.com/tfroehlich82/django-propeller" ' \
                 'class="navbar-brand navbar-brand-custome">propeller-test</a>' \
                 '</div>' \
                 '<div class="collapse navbar-collapse">' \
                 '<ul class="nav navbar-nav">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test1</a></li>' \
                 '<li class="dropdown pmd-dropdown"><a data-toggle="dropdown" ' \
                 'class="pmd-ripple-effect dropdown-toggle" data-sidebar="true" href="None">Test2' \
                 '<span class="caret"></span></a>' \
                 '<ul class="dropdown-menu">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test3</a></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test4</a></li>' \
                 '<li class="divider" role="separator"></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test5</a></li>' \
                 '</ul>' \
                 '</li>' \
                 '</ul>' \
                 '</div>' \
                 '</div>' \
                 '<div class="pmd-sidebar-overlay"></div>' \
                 '</nav>'

RESULT_NAVBAR2 = '<nav class="navbar navbar-inverse navbar-top pmd-navbar pmd-z-depth">' \
                 '<div class="container-fluid">' \
                 '<div class="navbar-header">' \
                 '<button aria-expanded="false" class="navbar-toggle collapsed" data-toggle="collapse" type="button">' \
                 '<span class="sr-only">Toggle navigation</span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '</button>' \
                 '<a href="javascript:void(0);" class="navbar-brand navbar-brand-custome">propeller-test</a>' \
                 '</div>' \
                 '<div class="collapse navbar-collapse">' \
                 '<ul class="nav navbar-nav">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test1</a></li>' \
                 '<li class="dropdown pmd-dropdown"><a data-toggle="dropdown" ' \
                 'class="pmd-ripple-effect dropdown-toggle" data-sidebar="true" href="None">Test2' \
                 '<span class="caret"></span></a>' \
                 '<ul class="dropdown-menu">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test3</a></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test4</a></li>' \
                 '<li class="divider" role="separator"></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test5</a></li>' \
                 '</ul>' \
                 '</li>' \
                 '</ul>' \
                 '</div>' \
                 '</div>' \
                 '<div class="pmd-sidebar-overlay"></div>' \
                 '</nav>'

RESULT_NAVBAR3 = '<nav class="navbar navbar-top pmd-navbar pmd-z-depth">' \
                 '<div class="container-fluid">' \
                 '<div class="navbar-header">' \
                 '<button aria-expanded="false" class="navbar-toggle collapsed" data-toggle="collapse" type="button">' \
                 '<span class="sr-only">Toggle navigation</span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '<span class="icon-bar"></span>' \
                 '</button>' \
                 '<a href="https://github.com/tfroehlich82/django-propeller" ' \
                 'class="navbar-brand navbar-brand-custome">propeller-test</a>' \
                 '</div>' \
                 '<div class="collapse navbar-collapse">' \
                 '<ul class="nav navbar-nav">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test1</a></li>' \
                 '<li class="dropdown pmd-dropdown">' \
                 '<a data-toggle="dropdown" class="pmd-ripple-effect dropdown-toggle" ' \
                 'data-sidebar="true" href="None">Test2<span class="caret"></span></a>' \
                 '<ul class="dropdown-menu">' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test3</a></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test4</a></li>' \
                 '<li class="divider" role="separator"></li>' \
                 '<li><a class="pmd-ripple-effect" href="javascript:void(0);">Test5</a></li>' \
                 '</ul>' \
                 '</li>' \
                 '</ul>' \
                 '</div>' \
                 '</div>' \
                 '<div class="pmd-sidebar-overlay"></div>' \
                 '</nav>'
