{% load propeller %}
{% for message in messages %}
    <div class="{{ message|propeller_message_classes }} alert-dismissable">
        <button type="button" class="close" data-dismiss="alert" aria-hidden="true">&#215;</button>
        {{ message }}
    </div>
{% endfor %}
